@@ -62,6 +62,12 @@ impl<'a> From<ArcBytes<'a>> for Bytes {
     }
 }
 
+impl<'a> From<Bytes> for ArcBytes<'a> {
+    fn from(bytes: Bytes) -> Self {
+        ArcBytes::owned(bytes.0)
+    }
+}
+
 impl Deref for Bytes {
     type Target = Vec<u8>;
 
@@ -99,6 +105,96 @@ impl<'de> Deserialize<'de> for Bytes {
     }
 }
 
+/// A `Cow<'a, [u8]>` wrapper that supports serializing efficiently in Serde.
+#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Default)]
+pub struct CowBytes<'a>(pub Cow<'a, [u8]>);
+
+impl<'a> CowBytes<'a> {
+    /// Returns the underlying Cow.
+    #[allow(clippy::missing_const_for_fn)] // false positive
+    #[must_use]
+    pub fn into_cow(self) -> Cow<'a, [u8]> {
+        self.0
+    }
+
+    /// Returns the underlying Vec inside of the Cow, or clones the borrowed bytes into a new Vec..
+    #[allow(clippy::missing_const_for_fn)] // false positive
+    #[must_use]
+    pub fn into_vec(self) -> Vec<u8> {
+        match self.0 {
+            Cow::Borrowed(bytes) => bytes.to_vec(),
+            Cow::Owned(vec) => vec,
+        }
+    }
+}
+
+impl<'a> From<Bytes> for CowBytes<'a> {
+    fn from(bytes: Bytes) -> Self {
+        CowBytes(Cow::Owned(bytes.0))
+    }
+}
+
+impl<'a> From<CowBytes<'a>> for Bytes {
+    fn from(bytes: CowBytes<'a>) -> Self {
+        match bytes.0 {
+            Cow::Borrowed(bytes) => Self(bytes.to_vec()),
+            Cow::Owned(vec) => Self(vec),
+        }
+    }
+}
+
+impl<'a> From<CowBytes<'a>> for ArcBytes<'a> {
+    fn from(bytes: CowBytes<'a>) -> Self {
+        ArcBytes::from(bytes.0)
+    }
+}
+
+impl<'a> From<Vec<u8>> for CowBytes<'a> {
+    fn from(buffer: Vec<u8>) -> Self {
+        Self(Cow::Owned(buffer))
+    }
+}
+
+impl<'a> From<&'a [u8]> for CowBytes<'a> {
+    fn from(buffer: &'a [u8]) -> Self {
+        Self(Cow::Borrowed(buffer))
+    }
+}
+
+impl<'a> Deref for CowBytes<'a> {
+    type Target = Cow<'a, [u8]>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl<'a> DerefMut for CowBytes<'a> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.0
+    }
+}
+
+impl<'a> Serialize for CowBytes<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        serializer.serialize_bytes(&self.0)
+    }
+}
+
+impl<'de: 'a, 'a> Deserialize<'de> for CowBytes<'a> {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        deserializer
+            .deserialize_byte_buf(BufferVisitor)
+            .map(CowBytes)
+    }
+}
+
 struct BufferVisitor;
 
 impl<'de> Visitor<'de> for BufferVisitor {
@@ -112,6 +208,7 @@ impl<'de> Visitor<'de> for BufferVisitor {
     where
         V: SeqAccess<'de>,
     {
+        println!("Sequence");
         let mut bytes = if let Some(len) = visitor.size_hint() {
             Vec::with_capacity(len)
         } else {
@@ -129,6 +226,7 @@ impl<'de> Visitor<'de> for BufferVisitor {
     where
         E: Error,
     {
+        println!("bytes");
         Ok(Cow::Owned(v.to_vec()))
     }
 
@@ -136,6 +234,7 @@ impl<'de> Visitor<'de> for BufferVisitor {
     where
         E: Error,
     {
+        println!("borrowed");
         Ok(Cow::Borrowed(v))
     }
 
@@ -143,6 +242,7 @@ impl<'de> Visitor<'de> for BufferVisitor {
     where
         E: Error,
     {
+        println!("byte_buf");
         Ok(Cow::Owned(v))
     }
 
@@ -150,6 +250,7 @@ impl<'de> Visitor<'de> for BufferVisitor {
     where
         E: Error,
     {
+        println!("str");
         Ok(Cow::Owned(v.as_bytes().to_vec()))
     }
 
@@ -157,6 +258,7 @@ impl<'de> Visitor<'de> for BufferVisitor {
     where
         E: Error,
     {
+        println!("borrowed str");
         Ok(Cow::Borrowed(v.as_bytes()))
     }
 
@@ -164,30 +266,35 @@ impl<'de> Visitor<'de> for BufferVisitor {
     where
         E: Error,
     {
+        println!("string");
         Ok(Cow::Owned(v.into_bytes()))
     }
 }
 
 #[test]
-fn tests() {
+fn serialization_tests() {
     use super::Bytes;
 
+    let simple_buffer = vec![1_u8, 2, 3];
+    let simple_buffer = simple_buffer.as_slice();
+    let simple_arcbytes = ArcBytes::from(simple_buffer);
+
     // deserialize_seq
-    let u8_sequence_bytes = pot::to_vec(&vec![1_u8, 2, 3]).unwrap();
+    let u8_sequence_bytes = pot::to_vec(&simple_buffer).unwrap();
     let buffer = pot::from_slice::<ArcBytes<'_>>(&u8_sequence_bytes).unwrap();
-    assert_eq!(buffer, &[1_u8, 2, 3]);
+    assert_eq!(buffer, simple_buffer);
     assert!(matches!(buffer.buffer, Bytes::Owned(_)));
 
     // deserialize_borrowed_bytes
-    let actual_bytes = pot::to_vec(&ArcBytes::from(vec![1_u8, 2, 3])).unwrap();
+    let actual_bytes = pot::to_vec(&simple_arcbytes).unwrap();
     let buffer = pot::from_slice::<ArcBytes<'_>>(&actual_bytes).unwrap();
-    assert_eq!(buffer, &[1_u8, 2, 3]);
+    assert_eq!(buffer, simple_buffer);
     assert!(matches!(buffer.buffer, Bytes::Borrowed(_)));
 
     // deserialize_byte_buf
-    let json = serde_json::to_string(&ArcBytes::from(vec![1_u8, 2, 3])).unwrap();
+    let json = serde_json::to_string(&simple_arcbytes).unwrap();
     let buffer = serde_json::from_str::<ArcBytes<'_>>(&json).unwrap();
-    assert_eq!(buffer, &[1_u8, 2, 3]);
+    assert_eq!(buffer, simple_buffer);
     assert!(matches!(buffer.buffer, Bytes::Owned(_)));
 
     // deserialize_str
@@ -202,7 +309,15 @@ fn tests() {
     assert!(matches!(buffer.buffer, Bytes::Owned(_)));
 
     // Deserialize `Bytes`
-    let actual_bytes = pot::to_vec(&ArcBytes::from(vec![1_u8, 2, 3])).unwrap();
+    let actual_bytes = pot::to_vec(&simple_arcbytes).unwrap();
     let buffer = pot::from_slice::<self::Bytes>(&actual_bytes).unwrap();
-    assert_eq!(buffer.as_slice(), &[1_u8, 2, 3]);
+    assert_eq!(buffer.as_slice(), simple_buffer);
+
+    // Deserialize `CowBytes`
+    let buffer = pot::from_slice::<self::CowBytes<'_>>(&actual_bytes).unwrap();
+    assert_eq!(&buffer[..], simple_buffer);
+    assert!(matches!(buffer.0, Cow::Borrowed(_)));
+    let buffer = pot::from_slice::<self::CowBytes<'_>>(&u8_sequence_bytes).unwrap();
+    assert_eq!(&buffer[..], simple_buffer);
+    assert!(matches!(buffer.0, Cow::Owned(_)));
 }
